unit PortDemo1;

{  This application demonstrates how to send a sequence of commands to a modem,
   wait for the modem to respond with OK, and then send next command. }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  uCiaComport, StdCtrls, ExtCtrls;

type
   TModemState = (msAT,
                  msATF,
                  msATE,
                  msAT2);

const
   aModemCmd: array[TModemState] of string = ('AT',
                                              'AT&F',
                                              'ATE1',
                                              'AT');

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Memo: TMemo;
    StartBtn: TButton;
    Com: TCiaComPort;
    TimeoutTimer: TTimer;
    procedure StartBtnClick(Sender: TObject);
    procedure ComDataAvailable(Sender: TObject);
    procedure TimeoutTimerTimer(Sender: TObject);
  private
    ModemState: TModemState;
    procedure Log(const ToLog: string);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------
procedure TForm1.Log(const ToLog: string);
var
   c: string;
begin
   c := Trim(ToLog);
   if c > '' then
      Memo.Lines.Add('> ' + c);
end;

//------------------------------------------------------------------------------
procedure TForm1.StartBtnClick(Sender: TObject);
begin
   ModemState           := TModemState(0);
   TimeoutTimer.Enabled := True;
   Com.Port             := 1;
   Com.LineEnd          := #13#10;
   Com.LineMode         := True;
   Com.Open             := True;
   Com.SendStr(aModemCmd[ModemState] + #13#10);
   StartBtn.Enabled     := False;
end;

//------------------------------------------------------------------------------
procedure TForm1.ComDataAvailable(Sender: TObject);
var
   Rcvd: string;
begin
   Rcvd := Com.ReceiveStr;
   Log(Rcvd);

   if Rcvd = 'OK'#13#10 then begin
      Inc(ModemState);
      TimeoutTimer.Enabled := False;
      if ModemState > High(TModemState) then begin
         Log('Modem initialized');
         StartBtn.Enabled := True;
         Exit;
      end;
      TimeoutTimer.Enabled := True;
      Com.SendStr(aModemCmd[ModemState] + #13#10);
   end;
end;

//------------------------------------------------------------------------------
procedure TForm1.TimeoutTimerTimer(Sender: TObject);
begin
   TimeoutTimer.Enabled := False;
   Com.Open             := False;
   StartBtn.Enabled     := True;
   Log('Modem timed out');
end;

end.
