unit uMain;

{  A Datascoop...

   Because many users of TCiaComport asked on how to write a serial port
   datascoop I decided to make a working example.

   Unlike DOS, many Windows OS does not permit to talk direcly with UART, so we
   have to do it by OS API. Problem is that windows does not give received bytes
   in real time, but typically if some timeout expires, or when (typically) 8
   bytes are received.

   A datascoop has to print at least data of both ends in the right sequence,
   if not it is useless. To bypass the problem we have to make our own timeout.
   Fortionaly we can deal with the way windows does preemptive multitasking.
   Typically a timeslice is 10 millisec for W2k, 13 millisec for NT workstation,
   20 millisec for NT server. So we can make the proposal, if a pheripharal
   device does not sent data during such a time, then it has send his packet.

   Most of the time (if not always) this proposal is correct. So we make a
   class witch we feed with received data and has a timeout on board. If it does
   not get data during a timeslice it fires an event to the application. See
   comment in uBuffer.pas.

   Code is made for speed and memory presume. Yes if allocating small amounts of
   memory it is very easy to fragment the heap and even get an 'out of memory'.
   For this reason all buffers are keeping data in a space that is re-used every
   time.

   I hope this example will help a lot of people who wants to build a datascopy,
   but also for others to serve as programming technique.

   best regards, Wilfried
   http://www.mestdagh.biz
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  uBuffer, StdCtrls, uCiaComport, ExtCtrls;

type
  TDTEDCE = (dte_buf, dce_buf);
  TForm1 = class(TForm)
    Panel1: TPanel;
    DTE: TComboBox;
    Label1: TLabel;
    DTEPort: TCiaComPort;
    DCE: TComboBox;
    Label2: TLabel;
    Activate: TCheckBox;
    Baud: TComboBox;
    Label3: TLabel;
    DCEPort: TCiaComPort;
    Memo: TMemo;
    DebugStr: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ActivateClick(Sender: TObject);
    procedure PortDataAvailable(Sender: TObject);
  private
    DTEBuf: TTimedBuffer;
    DCEBuf: TTimedBuffer;
    Debug: TDebug;
    procedure BufDataEnd(Sender: TObject);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
   CiaEnumPorts(DTE.Items);
   DCE.Items.Assign(DTE.Items);
   Debug            := TDebug.Create;
   DTEBuf           := TTimedBuffer.Create;
   DCEBuf           := TTimedBuffer.Create;
   DTEBuf.OnDataEnd := BufDataEnd;
   DCEBuf.OnDataEnd := BufDataEnd;
   DCEBuf.IsDCE     := True;
   DTEPort.Tag      := integer(DTEBuf);
   DCEPort.Tag      := integer(DCEBuf);
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   DTEPort.Open := False;
   DCEPort.Open := False;
   DTEBuf.Free;
   DCEBuf.Free;
   Debug.Free;
end;

procedure TForm1.ActivateClick(Sender: TObject);
begin
   with Sender as TCheckBox do try
      if Checked then begin
         if DTE.ItemIndex < 0 then
            raise Exception.Create('No DTE port');
         if DCE.ItemIndex < 0 then
            raise Exception.Create('No DCE port');
         if Baud.ItemIndex < 0 then
            raise Exception.Create('No Baudrate');
         if DTE.ItemIndex = DCE.ItemIndex then
            raise Exception.Create('DTE and DCE are on same port');
         DTEBuf.Clear;
         DCEBuf.Clear;
         DTEPort.Baudrate := StrToInt(Baud.Text);
         DTEPort.Port     := StrToInt(DTE.Text);
         DCEPort.Baudrate := DTEPort.Baudrate;
         DCEPort.Port     := StrToInt(DCE.Text);
      end;
      DTE.Enabled  := not Checked;
      DCE.Enabled  := not Checked;
      Baud.Enabled := not Checked;
      DTEPort.Open := Checked;
      DCEPort.Open := Checked;
   except
      Checked := False;
      raise;
   end;
end;

procedure TForm1.PortDataAvailable(Sender: TObject);
var
   Rcvd: array[0..127] of char;
   Count: integer;
begin
   {  We use same event for both ports. To know wich buffer to use, we use the
      Tag property which holds a pointer to the right buffer. Also we receive in
      a fixed buffer on the stack of 128 bytes. This is no problem because most
      of the time the amount of data available here will be only 8 bytes long. }

   Count := TCiaComPort(Sender).Receive(@Rcvd[0], 128);
   TTimedBuffer(TComponent(Sender).Tag).Write(Rcvd[0], Count);
end;

procedure TForm1.BufDataEnd(Sender: TObject);
var
   Buf: TTimedBuffer;
begin
   {  We use same event for both buffers. The buffers itself are set to deliver
      the right Header and Data. We dont need a copy of the data here, so we use
      Memory property. Because the buffer does not know then that we have read
      his data we have to Clear it ourself.  }

   Buf := TTimedBuffer(Sender);
   Memo.Lines.Add(Buf.Header);
   if DebugStr.Checked then begin
      Debug.DebugStr(Buf.Memory[0], Buf.Size);
      Memo.Lines.Add(Debug.Memory);
      Debug.Clear;
   end else
      Memo.Lines.Add(Buf.Memory);
   Buf.Clear;
   Memo.Lines.Add('');
end;

end.
